# -*- coding: UTF-8 -*-
"""
@Summary : docstr
@Author  : Rey
@Time    : 2022-04-01 17:03:32
"""
from enum import Enum

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.timezone import get_default_timezone, now as timezone_now
from django.utils.translation import gettext_lazy as _

from notice.models import NoticeType, ReceiverType
from notice.response import ValidationFailedDetailEnum
from notice.settings import NOTICE_DATETIME_FORMAT


class NoticeForm(forms.Form):

    class SendWayEnum(Enum):
        NO = 'no'
        NOW = 'now'
        TIMING = 'timing'

    SendWayChoices = (
        (SendWayEnum.NO.value, _('No')),
        (SendWayEnum.NOW.value, _('Now')),
        (SendWayEnum.TIMING.value, _('Timing')),
    )

    title = forms.CharField(required=False, max_length=64)
    content = forms.CharField(required=False, widget=forms.Textarea)
    type_id = forms.IntegerField(required=False, min_value=1)
    receiver_type_id = forms.IntegerField(required=False, min_value=1)
    send_way = forms.ChoiceField(required=True, choices=SendWayChoices)
    publish_at = forms.DateTimeField(required=False, input_formats=[NOTICE_DATETIME_FORMAT])

    def clean_type_id(self):
        type_id = self.cleaned_data['type_id']
        if not type_id:
            return type_id
        if not NoticeType.objects.filter(pk=type_id).exists():
            raise ValidationError(ValidationFailedDetailEnum.NOTICE_TYPE.value)
        return type_id

    def clean_receiver_type_id(self):
        receiver_type_id = self.cleaned_data['receiver_type_id']
        if not receiver_type_id:
            return receiver_type_id
        if not ReceiverType.objects.filter(pk=receiver_type_id).exists():
            raise ValidationError(ValidationFailedDetailEnum.RECEIVER_TYPE.value)
        return receiver_type_id

    def clean_publish_at(self):
        send_way = self.cleaned_data.get('send_way')
        if send_way is None:
            raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)
        if send_way == self.SendWayEnum.NO.value:
            self.cleaned_data['is_draft'] = True
            return None
        if send_way == self.SendWayEnum.NOW.value:
            self.cleaned_data['is_draft'] = False
            return timezone_now()
        if send_way == self.SendWayEnum.TIMING.value:
            self.cleaned_data['is_draft'] = False
            publish_at = self.cleaned_data.get('publish_at')
            if not publish_at:
                raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)
            if settings.USE_TZ:
                if publish_at.replace(tzinfo=get_default_timezone()) <= timezone_now():
                    raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
            else:
                if publish_at <= timezone_now():
                    raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
            return publish_at
        raise ValidationError(ValidationFailedDetailEnum.SEND_WAY.value)


class ChangeTimingForm(forms.Form):
    publish_at = forms.DateTimeField(required=True, input_formats=[NOTICE_DATETIME_FORMAT])

    def clean_publish_at(self):
        publish_at: str = self.cleaned_data.get('publish_at')
        if not publish_at:
            raise ValidationError(ValidationFailedDetailEnum.PUBLISH_TIME.value)

        if settings.USE_TZ:
            if publish_at.replace(tzinfo=get_default_timezone()) <= timezone_now():
                raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
        else:
            if publish_at <= timezone_now():
                raise ValidationError(ValidationFailedDetailEnum.OUTDATE.value)
        return publish_at



# -*- coding: UTF-8 -*-
"""
@Summary : orm
@Author  : Rey
@Time    : 2022-03-30 11:44:57
"""
from enum import Enum

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from notice.settings import NOTICE_DATETIME_FORMAT


class BaseTimeModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_('create time'))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_('latest update time'))

    class Meta:
        abstract = True


class IsDeletedManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_deleted=False)


class IsDeletedModel(models.Model):
    is_deleted = models.BooleanField(default=False, verbose_name=_('delete tag'))

    objects = IsDeletedManager()
    all_objects = models.Manager()
    class Meta:
        abstract = True


class NameDescModel(models.Model):
    name = models.CharField(max_length=64, verbose_name=_('name'))
    desc = models.CharField(max_length=256, verbose_name=_('description'))
    class Meta:
        abstract = True


class ReceiverType(BaseTimeModel, IsDeletedModel, NameDescModel):
    class Meta:
        db_table = 'notice_receiver_type'


class NoticeType(BaseTimeModel, IsDeletedModel, NameDescModel):
    class Meta:
        db_table = 'notice_type'


class NoticeStore(BaseTimeModel, IsDeletedModel):
    title = models.CharField(null=True, max_length=64, verbose_name=_('title'))
    content = models.TextField(null=True, verbose_name=_('content'))
    notice_type = models.ForeignKey(NoticeType, on_delete=models.CASCADE, verbose_name=_('notice type'))
    receiver_type = models.ForeignKey(ReceiverType, on_delete=models.CASCADE, verbose_name=_('receiver type'))
    is_draft = models.BooleanField(default=True, verbose_name=_('draft tag'))
    creator_id = models.IntegerField(verbose_name=_('creator id'))
    publish_at = models.DateTimeField(null=True, verbose_name=_('publish time'))

    class StatusEnum(Enum):
        DRAFT = 'draft'
        QUEUE = 'queue'
        DONE = 'done'

    StatusLabel = {
        StatusEnum.DRAFT: _('Draft'),
        StatusEnum.QUEUE: _('Queue'),
        StatusEnum.DONE: _('Done'),
    }

    @property
    def status(self):
        if self.is_draft:
            return self.StatusEnum.DRAFT
        if self.publish_at > timezone.now():
            return self.StatusEnum.QUEUE
        return self.StatusEnum.DONE

    @property
    def published_at(self):
        if not self.is_draft and self.publish_at <= timezone.now():
            return self.publish_at.strftime(NOTICE_DATETIME_FORMAT)
        return None


class ReceiverTag(BaseTimeModel, IsDeletedModel):
    noticestore = models.OneToOneField(NoticeStore, on_delete=models.CASCADE, verbose_name=_('notice'))
    receiver_id = models.IntegerField(verbose_name=_('receiver'))
    read_at = models.DateTimeField(verbose_name=_('read time'))

    class Meta:
        db_table = 'notice_receiver_tag'
